@@ -4,7 +4,7 @@
 *  Based upon CGI/Ex/Validate.pm v0.94 from Perl                     *
 *  For instructions on usage, see perldoc of CGI::Ex::Validate       *
 ***----------------------------------------------------------------**/
-// $Revision: 1.20 $
+// $Revision: 1.24 $
 
 function Validate () {
   this.error             = vob_error;
@@ -184,6 +184,7 @@ function vob_check_conditional (form, ifs, N_level, ifs_match) {
   /// run the if options here
   /// multiple items can be passed - all are required unless OR is used to separate
   var is_found = 1;
+  var m;
   for (var i = 0; i < ifs.length; i ++) {
     var ref = ifs[i];
     if (typeof(ref) != 'object') {
@@ -194,8 +195,13 @@ function vob_check_conditional (form, ifs, N_level, ifs_match) {
       } else {
         var field = ref;
         ref = new Array();
+        if (m = field.match('^(\\s*!\\s*)')) {
+          field = field.substring(m[1].length);
+          ref['max_in_set'] = '0 of ' + field;
+        } else {
+          ref['required'] = 1;
+        }
         ref['field'] = field;
-        ref['required'] = 1;
       }
     }
     if (! is_found) break;
@@ -273,6 +279,16 @@ function vob_validate_buddy (form, field, field_val, N_level, ifs_match) {
   }
   var n_values = (typeof(_value) == 'undefined') ? 0 : values.length;
 
+  /// allow for default value
+  var tests = this.filter_types('default', types);
+  if (n_values == 0 || (n_values == 1 && values[0].length == 0)) {
+    for (var i = 0; i < tests.length; i ++) {
+      var el = form[field];
+      var type = el.type;
+      if (type && (type == 'hidden' || type == 'password' || type == 'text' || type == 'textarea' || type == 'submit')) el.value = values[0] = field_val[tests[i]];
+    }
+  }
+
   /// allow for a few form modifiers
   var modified = 0;
   for (var i = 0; i < values.length; i ++) {
@@ -288,7 +304,7 @@ function vob_validate_buddy (form, field, field_val, N_level, ifs_match) {
   var tests = this.filter_types('replace', types);
   for (var i = 0; i < tests.length; i ++) {
     var ref = field_val[tests[i]];
-    ref = (typeof(ref) == 'object') ? ref : ref.split('\\s*\\|\\|\\s*');
+    ref = (typeof(ref) == 'object') ? ref : ref.split(new RegExp('\\s*\\|\\|\\s*'));
     for (var j = 0; j < ref.length; j ++) {
       if (! (m = ref[j].match('^\\s*s([^\\s\\w])(.+)\\1(.*)\\1([eigmx]*)$')))
         return this.error("Not sure how to parse that replace "+ref[j]);
@@ -361,7 +377,7 @@ function vob_validate_buddy (form, field, field_val, N_level, ifs_match) {
   }
 
   /// max values check
-  var tests = this.filter_types('min_values', types);
+  var tests = this.filter_types('max_values', types);
   if (! tests.length) {
     tests[tests.length] = 'max_values';
     field_val['max_values'] = 1;
@@ -374,6 +390,39 @@ function vob_validate_buddy (form, field, field_val, N_level, ifs_match) {
     }
   }
 
+  /// min_in_set and max_in_set check
+  for (var h = 0; h < 2 ; h++) {
+    var minmax = (h == 0) ? 'min' : 'max';
+    var tests = this.filter_types(minmax+'_in_set', types);
+    for (var i = 0; i < tests.length; i ++) {
+      if (! (m = field_val[tests[i]].match('^\\s*(\\d+)(?:\\s*[oO][fF])?\\s+(.+)\\s*$')))
+        return this.error("Invalid in_set check "+field_val[tests[i]]);
+      var n       = m[1];
+      var _fields = m[2].split(new RegExp('[\\s,]+'));
+      for (var k = 0; k < _fields.length; k ++) {
+        var _value = this.get_form_value(form[_fields[k]]);
+        var _values;
+        if (typeof(_value) == 'undefined') continue;
+        if (typeof(_value) == 'object') {
+          _values = _value;
+        } else {
+          _values = new Array();
+          _values[_values.length] = _value;
+        }
+        for (var l = 0; l < _values.length; l ++) {
+          var _value = _values[l];
+          if (typeof(_value) != 'undefined' && _value.length) n --;
+        }
+      }
+      if (   (minmax == 'min' && n > 0)
+          || (minmax == 'max' && n < 0)) {
+        this.add_error(errors, field, tests[i], field_val, ifs_match);
+        return errors;
+      }
+    }
+  }
+
+  // the remaining tests operate on each value of a field
   for (var n = 0; n < values.length; n ++) {
     var value = values[n];
     
@@ -381,7 +430,7 @@ function vob_validate_buddy (form, field, field_val, N_level, ifs_match) {
     var tests = this.filter_types('enum', types);
     for (var i = 0; i < tests.length; i ++) {
       var hold  = field_val[tests[i]];
-      var _enum = (typeof(hold) == 'object') ? hold : hold.split('\\s*\\|\\|\\s*');
+      var _enum = (typeof(hold) == 'object') ? hold : hold.split(new RegExp('\\s*\\|\\|\\s*'));
       var is_found = 0;
       for (var j = 0; j < _enum.length; j ++) {
         if (value != _enum[j]) continue;
@@ -426,7 +475,7 @@ function vob_validate_buddy (form, field, field_val, N_level, ifs_match) {
       var ref = field_val[tests[i]];
       ref = (typeof(ref) == 'object') ? ref
         : (typeof(ref) == 'function') ? new Array(ref)
-        : ref.split('\\s*\\|\\|\\s*');
+        : ref.split(new RegExp('\\s*\\|\\|\\s*'));
       for (var j = 0; j < ref.length; j ++) {
         if (typeof(ref[j]) == 'function') {
           if (! value.match(ref[j])) this.add_error(errors, field, tests[i], field_val, ifs_match);
@@ -452,7 +501,7 @@ function vob_validate_buddy (form, field, field_val, N_level, ifs_match) {
     var tests = this.filter_types('compare', types);
     for (var i = 0; i < tests.length; i ++) {
       var ref = field_val[tests[i]];
-      ref = (typeof(ref) == 'object') ? ref : ref.split('\\s*\\|\\|\\s*');
+      ref = (typeof(ref) == 'object') ? ref : ref.split(new RegExp('\\s*\\|\\|\\s*'));
       for (var j = 0; j < ref.length; j ++) {
         var comp = ref[j];
         if (! comp) continue;
@@ -526,7 +575,7 @@ function vob_check_type (value, type, field, form) {
   /// standard IP address
   } else if (type == 'IP') {
     if (! value) return 0;
-    var dig = value.split('\\.');
+    var dig = value.split(new RegExp('\\.'));
     if (dig.length != 4) return 0;
     for (var i = 0; i < 4; i ++)
       if (typeof(dig[i]) == 'undefined' || dig[i].match('\\D') || dig[i] > 255) return 0;
@@ -582,13 +631,17 @@ function vob_check_type (value, type, field, form) {
 // little routine that will get the values from the form
 // it will return multiple values as an array
 function vob_get_form_value (el) {
-   if (! el) return '';
+  if (! el) return '';
   if (el.disabled) return '';
   var type = el.type ? el.type.toLowerCase() : '';
-  if (el.length && el.type != 'select-one') {
+  if (el.length && type != 'select-one') {
     var a = new Array();
     for (var j=0;j<el.length;j++) {
-      if (el[j].checked) a[a.length] = vob_get_form_value(el[j]);
+      if (type.indexOf('multiple')) {
+        if (el[j].selected) a[a.length] = el[j].value;
+      } else {
+        if (el[j].checked)  a[a.length] = vob_get_form_value(el[j]);
+      }
     }
     if (a.length == 0) return '';
     if (a.length == 1) return a[0];
@@ -597,7 +650,7 @@ function vob_get_form_value (el) {
   if (! type) return '';
   if (type == 'hidden' || type == 'password' || type == 'text' || type == 'textarea' || type == 'submit')
     return el.value;
-  if (type == 'select-one') {
+  if (type.indexOf('select') != -1) {
     if (! el.length) return '';
     return el[el.selectedIndex].value;
   }
@@ -790,7 +843,16 @@ function eob_get_error_text (err) {
     var n = field_val["max_values" + dig];
     var values = (n == 1) ? 'value' : 'values';
     return name + " had more than "+n+" "+values+".";
-  
+
+  } else if (type == 'min_in_set') {
+    var set = field_val["min_in_set" + dig];
+    return "Not enough fields were chosen from the set ("+set+")";
+    return "Too many fields were chosen from the set ("+set+")";
+
+  } else if (type == 'max_in_set') {
+    var set = field_val["max_in_set" + dig];
+    return "Too many fields were chosen from the set ("+set+")";
+
   } else if (type == 'enum') {
     return name + " is not in the given list.";
   
